function [Z_right,Z_left,VI2] = AfterMatching( Freq, Z_right_initial,Z_left_initial,Components_Matrix)

[row,column] = size(Components_Matrix);
[x,y] = size(Freq);

Z_right = Z_right_initial; % Orgininal Value of Coil's raw impedance

a_inital = ones(2,2,x);

%% Compute Z_right, impedance that Opamp sees
for i = 1:row
    if Components_Matrix(i,2) == 0 %L
        % 0 for L
        Z_component = 1i*2*pi*Freq * Components_Matrix(i,1);
    elseif Components_Matrix(i,2) == 1 %C
        % 1 for C
        Z_component = 1./ (1i*2*pi*Freq * Components_Matrix(i,1) ) ;
    elseif Components_Matrix(i,2) == 2 %tranformer 
        Z_component = Components_Matrix(i,1); %impedance ratio
    elseif Components_Matrix(i,2) == 3 %TL
        Z0 = Components_Matrix(i,1); %Z0 of TL
    elseif Components_Matrix(i,2) == 4 % Resistor
        Z_component = zeros(x,1);
        Z_component(:,1) = Components_Matrix(i,1); % R
    elseif Components_Matrix(i,2) == 5 % LC-C Trap( L2C2 parallel, series C1)
        Z_C1 =  1./ (1i*2*pi*Freq * Components_Matrix(i,1) ) ; % C1
        Z_C2 = 1./ (1i*2*pi*Freq * Components_Matrix(i,4) ) ; % C2
        Z_L2 =  0.2 + 1i*2*pi*Freq * Components_Matrix(i,5); % L2
        Z_component = Z_C1 + 1./( (1./Z_C2) + (1./Z_L2) );
    elseif Components_Matrix(i,2) == 6 % LC-C Trap( L2C3 series para with C1)
        Z_C1 =  1./ (1i*2*pi*Freq * Components_Matrix(i,1) ) ; % C1
        Z_C2 = 1./ (1i*2*pi*Freq * Components_Matrix(i,4) ) ; % C2
        Z_L2 = 0.2 + 1i*2*pi*Freq * Components_Matrix(i,5); % L2
        Z_component = 1./( (1./Z_C1) + (1./(Z_C2+Z_L2)) );
    else 
        error('2nd column wrong, L, C, Tfs or TL')
    end
    
    if Components_Matrix(i,3) == 0 %Series 
        % 0 for series
        Z_right = Z_right + Z_component;
        for j = 1:x
            a(:,:,j,i) = [ 1 Z_component(j) ; 0 1 ] ;
        end
    elseif Components_Matrix(i,3) == 1 %shunt 
        % 1 for Parallel
        Z_right = 1 ./ ( (1./Z_right) + (1./Z_component) );
        for j = 1:x
            a(:,:,j,i) = [ 1 0 ; 1/Z_component(j) 1 ] ;
        end
    elseif Components_Matrix(i,3) == 2 %transformer 
        Z_right = Z_right * Z_component;
        for j = 1:x
            a(:,:,j,i) = [ 1/sqrt(Z_component) 0 ; 0 sqrt(Z_component) ] ;
        end
    elseif Components_Matrix(i,3) == 3% TL
        L_Phy = Components_Matrix(i,4);
        Z_right = TransmissionLine_Z( Z0, L_Phy, Freq, Z_right);
        for j = 1:x
            Er = 2.23;
            Lambda = 3e8 ./ (Freq(j)*sqrt(Er)); %lambda
            yl = 1i * 2*pi * (L_Phy/Lambda ) ;
            a(:,:,j,i) = [ cosh(yl) Z0*sinh(yl) ; sinh(yl)/Z0 cosh(yl)] ;
        end
    else
        error('3rd column wrong, series, shunt, Tfs or TL')
    end    
end

%% comput Z_left, the impedance that Coil sees
Z_left = Z_left_initial; % Orgininal Value
Components_Matrix = flipud(Components_Matrix);
for i = 1:row
    if Components_Matrix(i,2) == 0
        % 0 for L
        Z_component = 1i*2*pi*Freq * Components_Matrix(i,1);
    elseif Components_Matrix(i,2) == 1
        % 1 for C
        Z_component = 1./ ( 1i*2*pi*Freq * Components_Matrix(i,1) ) ;
    elseif Components_Matrix(i,2) == 2 
        %tranformer 
        Z_component = 1 / Components_Matrix(i,1); % inversed here
    elseif Components_Matrix(i,2) == 3 
        %TL
        Z0 = Components_Matrix(i,1); %Z0 of TL
    elseif Components_Matrix(i,2) == 4 % Resistor
        Z_component = Components_Matrix(i,1); % R
    elseif Components_Matrix(i,2) == 5 % LC-C Trap( L2C2 parallel, series C1)
        Z_C1 =  1./ (1i*2*pi*Freq * Components_Matrix(i,1) ) ; % C1
        Z_C2 = 1./ (1i*2*pi*Freq * Components_Matrix(i,4) ) ; % C2
        Z_L2 = 1i*2*pi*Freq * Components_Matrix(i,5); % L2
        Z_component = Z_C1 + 1./( (1./Z_C2) + (1./Z_L2) );
    elseif Components_Matrix(i,2) == 6 % LC-C Trap( L2C3 series para with C1)
        Z_C1 =  1./ (1i*2*pi*Freq * Components_Matrix(i,1) ) ; % C1
        Z_C2 = 1./ (1i*2*pi*Freq * Components_Matrix(i,4) ) ; % C2
        Z_L2 = 1i*2*pi*Freq * Components_Matrix(i,5); % L2
        Z_component = 1./( (1./Z_C1) + (1./(Z_C2+Z_L2)) );
    else 
        error('2nd column wrong, L, C, Tfs or TL')
    end
    
    if Components_Matrix(i,3) == 0
        % 0 for series
        Z_left = Z_left + Z_component;
    elseif Components_Matrix(i,3) == 1
        % 1 for Parallel
        Z_left = 1 ./ ( (1./Z_left) + (1./Z_component) );
    elseif Components_Matrix(i,3) == 2 %transformer 
        Z_left = Z_left * Z_component;
    elseif Components_Matrix(i,3) == 3 % TL
        L_Phy = Components_Matrix(i,4);
        Z_left = TransmissionLine_Z( Z0, L_Phy, Freq, Z_left);
    else
        error('3rd column wrong, series, shunt, Tfs or TL')
    end    
end

% find V1 and I1 for abcd matrix calulcation
Vsource= 1;
I1 = Vsource ./ ( Z_right_initial + Z_left ); 
V1 = I1 .* Z_left; 
%Coil's inital impedance and the impedance coil see 
% Freq_27MHZ_Index =  34; %17O 27.1284MHz
% Freq_30MHZ_Index =  40; %2H 30.7192 MHZ
% Freq_50MHZ_Index =  78;  %13C, 50.3294MHZ
% Freq_53MHZ_Index =  83;  %23Na
% Freq_81MHZ_Index =  136;  %31P, 81.0045MHZ
% Freq_188MHZ_Index =  341;  %19F, 188.24MHZ
% Freq_200MHZ_Index = 363; %1H
% Index = [34,40,78,83,136,341,363];
for j = 1:x %freq count
    if row == 1
        a_cascaded(:,:,j) = a(:,:,j,1);
    else
        a_cascaded(:,:,j) = a(:,:,j,1);
        for i = 1:(row-1) % elements number count
            a_cascaded(:,:,j) = a_cascaded(:,:,j) * a(:,:,j,i+1);  
        end
    end
    
    if j ==363
       abcd = 1; 
    end
    
    VI1(:,:,j) = [ V1(j) ; I1(j)];
    VI2(:,:,j) = a_cascaded(:,:,j) \ VI1(:,:,j) ;
end


end