function [NF] = NF_Calculation_LMH6629_NonInverting(Rf,Rg,R_s)
% NF of a non-inverting OPA is calculated based on 3 resistors value: 
% Source Resisance(Rs), Feedback Resistance(Rf) and Shunt Resistance(Rg)
% Eqution are all come from TI's document :
% https://www.ti.com/lit/an/slyt094/slyt094.pdf?ts=1618261747981&ref_url=https%253A%252F%252Fwww.google.com%252F

% R_s = 10:1:10e3; 
% [A,B] = size(R_s);

k=1.38e-23;
T=300;
G = (Rf/Rg) + 1;
% Rf = 100;
% Rg = Rf/(G-1);
Req = 1/( (1/Rf) + (1/Rg) );
BW_3dB= 300e6;

en = 0.69e-9;
in = 2.6e-12 ;

%% Broadband Noise Component RMS:
BWn = 1.57 * BW_3dB ;
Enbb = en*sqrt(BWn) ;

%% 1/f Noise Component RMS:
Enormal = 1.3e-9 * sqrt(0.1);
Eflicker = Enormal*sqrt(log(BWn/0.1));

%% Total op-amp Voltage noise referred to the input RMS:
En_v = sqrt(Enbb^2+Eflicker^2);

%% Translating Current to Voltage Spectral Density:
en_i = in * (Req+R_s);

%% Convert Spec density to RMS
En_i = en_i * sqrt(BWn);

%% Resistor Thermol Noise RMS
En_r = sqrt(4*k*T*Req*BWn);

%% Total Noise RTI RMS:
Ent_in = sqrt(En_v^2+En_i.^2+En_r^2);

%% Total Noise RTO RMS:
Ent_out = Ent_in*G;

%% Convert RMS to PP:
Ent_out_pp = Ent_out*6;

%% Source impedance Noise RMS:
% R_s = 10:10:5e3; 
En_r_s = sqrt(4*k*T*R_s*BWn);

%% NF vs Source resistance

NF = 10*log10(1+ (Ent_in./En_r_s).^2 );

%% Ploting

% En_v_matrix(1,1:B) = En_v; 
% En_r_matrix(1,1:B) = En_r; 
% 
% En_in_total = sqrt(Ent_in.^2+En_r_s.^2);
% 
% 
% figure
% hold on
% semilogx(R_s,En_v_matrix*1e6)
% semilogx(R_s,En_i*1e6)
% semilogx(R_s,En_r_matrix*1e6)
% semilogx(R_s,Ent_in*1e6)
% semilogx(R_s,En_r_s*1e6)
% semilogx(R_s,En_in_total*1e6)
% legend('Opa voltage noise','Opa current noise','Opa resistor noise','Opa Total Noise','Source Noise','total noise')
% title('Input referred Noise voltage level for different source')
% xlabel('Source Resistor in Ohm')
% ylabel('Noise Voltage in uV, BW-471MHz')
% hold off
% 
% figure
% 
% semilogx(R_s,NF,'LineWidth',2)
% title('Calculated Noise Figure of OPA vs Different Source Resistance')
% xlabel('Source Resistance in Ohm')
% ylabel('NF in dB')
% grid on



end